from setuptools import setup, find_packages

setup(
    name="interpretability-core",
    version="0.1.2",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    install_requires=[
        "torch",
        "transformers",
        "pandas",
        "pyyaml",
        "pillow",
        "tqdm",
        # ... other dependencies
    ],
    python_requires=">=3.10",
)